@@ -39,16 +39,23 @@ $.fn.postprocess = function (className, options) {
                 var data = $(this).attr('resty_value');
                 var type = $(this).attr('resty_type');
                 //debug(type);
-                if ((data && data.length > 30) || /\n/.test(data)) {
+                if (!type) {
+                    type = 'text';
+                } 
+                
+                if (type != 'select' && ((data && data.length > 30) || /\n/.test(data))) {
                     type = 'textarea';
                 }
-                if (!type) type = 'text';
                 if (type == 'textarea') {
                     //debug("found textarea!");
                     settings.width = 600;
                     settings.height = 200;
-                } else {
+                } else if (type == 'text') {
                     settings.width = data ? (data.length + 5 + 'em') : '5em';
+                } else if (type == 'select') {
+                    settings.width = '10em';
+                } else {
+            
                 }
                 settings.data = data;
                 settings.type = type;
@@ -74,6 +81,9 @@ function plantEditableHook (node, settings) {
             }
             value = res;
         }
+        if (key == 'unique' || key == 'not_null') {
+            value = (value == 'true' ? true: false);
+        }
         data[key] = value;
         //debug("PUT /=/" + path + " " + JSON.stringify(data));
         setStatus(true, 'editInplace');
@@ -581,17 +591,26 @@ function getColumnSpec (container) {
     var col = {};
     var found = false;
     //debug(div);
-    $("input.column-input", container).each( function () {
+    $(".column-input", container).each( function () {
         var key = $(this).attr('resty_key');
         //debug("Key: " + key);
         var val = $(this).val();
         if (!key) return;
+        // alert(key);
+        /* default expression doesn't need parse *
         if (key == 'default' && val) {
             var res = JSON.parse(val);
             if (res == false && typeof res == typeof false && value != false) {
                 throw("Invalid JSON for the column's default value: " + val);
             }
             val = res;
+        } */
+        if (key == "unique" || key == "not_null") {
+            if (val == 'true'){
+                val = true;
+            } else {
+                val = false;
+            }
         }
         if (val != '') {
             col[key] = val;
