@@ -1,6 +1,6 @@
 package OpenResty;
 
-our $VERSION = '0.005002';
+our $VERSION = '0.005003';
 
 use strict;
 use warnings;
@@ -660,7 +660,7 @@ OpenResty - General-purpose web service platform for web applications
 
 =head1 VERSION
 
-This document describes OpenResty 0.5.2 released on November 27, 2008.
+This document describes OpenResty 0.5.3 released on November 30, 2008.
 
 =head1 DESCRIPTION
 
@@ -825,6 +825,39 @@ generated by L<Module::Install> for CPAN building system.
 
 =back
 
+=head1 PERFORMANCE
+
+OpenResty takes runtime performance very seriously because we have to run it on our not-so-good servers and support lots of Yahoo! China's online products with very heavy traffic.
+
+OpenResty prefers modules with XS over pure Perl ones and use cache aggressively. It's also in favor of source-filter based solutions provided by L<Filter::QuasiQuote> to reduce the length of subroutine calling chains and the number of indirections. Finally, the restyscript compiler is also written in carefully optimized Haskell code to maximize speed.
+
+The benchmark results for OpenResty 0.5.3's test suite on a Pentium 4 3.0GHz machine is given below:
+
+=over
+
+=item in-process frontend + PgMocked backend
+
+    DELETE: 4 ms (157 trials)
+    POST: 23 ms (493 trials)
+    PUT: 5 ms (132 trials)
+    GET: 4 ms (648 trials)
+
+=item lighttpd fastcgi frontend + local Pg backend
+
+    DELETE: 29 ms (193 trials)
+    POST: 30 ms (815 trials)
+    PUT: 11 ms (138 trials)
+    GET: 9 ms (763 trials)
+
+=item lighttpd fastcgi frontend + remote PgFarm backend
+
+    DELETE: 99 ms (193 trials)
+    POST: 98 ms (815 trials)
+    PUT: 41 ms (138 trials)
+    GET: 24 ms (763 trials)
+
+=back
+
 =head1 SOURCE CONTROL
 
 For the very latest version of this module, check out the source from
@@ -835,6 +868,44 @@ L<http://svn.openfoundry.org/openapi/trunk>
 There is anonymous access to all. If you'd like a commit bit, please let
 us know. :)
 
+=head1 IRC Channel
+
+Join us via IRC on C<#openresty> of the irc.freenode.net server ;)
+
+=head1 Project Roadmap
+
+Below is a list of currently planned release milestones (but it's also supposed to change as we go):
+
+=over
+
+=item 0.5.x (Where we are)
+
+Action API and an enhanced version of the Model API.
+
+=item 0.6.x
+
+Migrate the View handler to the same style and implementation of the Action handler, i.e., using explicit parameter list and taking advantage of the Haskell version of the restyscript compiler.
+
+Compiling view definition to native PostgreSQL functions is also supposed to realize in this series.
+
+=item 0.7.x
+
+Attachment API, which supports binary file uploading and downloading.
+
+=item 0.8.x
+
+Mail API, which introduces builtin Models for email sentbox and inbox based on third-party POP3/STMP servers.
+
+It will also allow actions to be triggered and/or confirmed by emails.
+
+=item 0.9.x
+
+Prophet/Git integration.
+
+=back
+
+Please don't hesitate to tell us what you love to see in future releases of OpenResty ;)
+
 =head1 TODO
 
 For the project's TODO list, please check out L<http://svn.openfoundry.org/openapi/trunk/TODO>
@@ -866,21 +937,30 @@ For a complete list of the contributors, please see L<http://svn.openfoundry.org
 
 =head1 License and Copyright
 
-Copyright (C) 2007-2008   Yahoo! China EEEE Works, Alibaba Inc
+OpenResty is licensed under the BSD License:
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+Copyright (c) 2007-2008, Yahoo! China EEEE Works, Alibaba Inc.
+All rights reserved.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
-You should have received a copy of the GNU General Public License along
-with this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+=over
+
+=item *
+
+Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+
+=item *
+
+Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
+
+=item *
+
+Neither the name of the Yahoo! China EEEE Works, Alibaba Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
+
+=back
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =head1 SEE ALSO
 
