@@ -1,12 +1,89 @@
 package OpenResty::Handler::Login;
 
-#use Smart::Comments;
+#use Smart::Comments '####';
 use strict;
 use warnings;
 
 use CGI::Simple::Cookie;
 use OpenResty::Util;
 use Params::Util qw( _STRING );
+use OpenResty::Handler::Logout;
+use OpenResty::QuasiQuote::SQL;
+
+use base 'OpenResty::Handler::Base';
+
+__PACKAGE__->register('login');
+
+sub requires_acl { undef }
+
+sub level2name {
+    qw< login login_user login_user_password >[$_[-1]]
+}
+
+sub login_per_request {
+    my ($class, $openresty, $bits) = @_;
+    my ($account, $role);
+    # XXX this part is lame...
+    # XXX param user is now deprecated; use _user instead
+    my $session = $openresty->get_session;
+    my $user = $openresty->builtin_param('_user');
+    #warn "_user = $user\n";
+    if (defined $user) {
+        #$OpenResty::Cache->remove($uuid);
+        my $captcha = $openresty->builtin_param('_captcha');
+        ### URL param capture: $captcha
+        #require OpenResty::Handler::Login;
+        my $res = OpenResty::Handler::Login->login($openresty, $user, {
+            password => $openresty->builtin_param('_password'),
+            captcha => $captcha,
+        });
+        $account = $res->{account};
+        $role = $res->{role};
+        # XXX login as $account.$role...
+        # XXX if account is anonymous, then create a session
+        # XXX else check password, if correct, create a session
+    } else {
+        if ($session) {
+            my $user = $OpenResty::Cache->get($session);
+            #### User from cache: $user
+            if ($user) {
+                ($account, $role) = split /\./, $user, 2;
+            }
+            #### $account
+            ### $role
+        }
+    }
+
+    my $call_level = $openresty->{_call_level};
+    if ($call_level == 0) {
+        # this part is lame?
+        if (!$account) {
+            die "Login required.\n";
+        }
+        if (!$openresty->has_user($account)) {
+            ### Found user: $user
+            die "Account \"$account\" does not exist.\n";
+        }
+    } else {
+        if (!$account) {
+            $account = $openresty->{_parent_account};
+        } else {
+            if (!$openresty->has_user($account)) {
+                ### Found user: $user
+                die "Account \"$account\" does not exist.\n";
+            }
+        }
+    }
+    #warn "Setting user: $account ", join ('/', @$bits), "\n";
+    $openresty->set_user($account);
+
+    $role ||= 'Admin';
+    if (!$openresty->has_role($role)) {
+        ### Found user: $user
+        die "Role \"$role\" does not exist.\n";
+    }
+    $openresty->set_role($role);
+}
 
 #*login = \&login_by_sql;
 *login = \&login_by_perl;
@@ -64,12 +141,13 @@ sub login_by_perl {
     ### $role
     ### $password
     ### capture param:  $captcha
+    #warn "!!!!!!!!!!!!!!!!!!! Captcha Value: $captcha\n";
     if (defined $captcha) {
+        #warn "!!!!!!!!!!!!!!!!!!! Captcha Value: $captcha\n";
         my ($id, $user_sol) = split /:/, $captcha, 2;
         if (!$id or !$user_sol) {
             die "Bad captcha parameter: $captcha\n";
         }
-        ### with captcha: $res
         #warn "!!! $login_meth !!!\n";
         if ($login_meth ne 'captcha') {
             die "Cannot login as $account.$role via captchas.\n";
@@ -80,12 +158,19 @@ sub login_by_perl {
             die $err."\n";
         }
     } elsif (defined $password) {
-        if (!$login_meth eq 'password') {
+        #warn "Login meth: $login_meth\n";
+        if ($login_meth ne 'password') { # short-cut
+            #warn "HERE!!!";
             die "Password for $account.$role is incorrect.\n";
         }
-        my $res = $openresty->select("select count(*) from _roles where name = " . Q($role) . " and password = " . Q($password) . ";");
-        ### with password: $res
-        if ($res->[0][0] == 0) {
+        my $res = $openresty->select([:sql|
+            select id
+            from _roles
+            where name = $role and password = $password
+            limit 1;
+        |]);
+        #### with password: $res
+        if (! @$res) {
             die "Password for $account.$role is incorrect.\n";
         }
     } else {
